@@ -563,16 +563,20 @@ ParameterMerger = Callable[[Iterable[Parameter]], Parameter]
 conservative_parameter_merge: ParameterMerger
 
 
-def conservative_parameter_merge(params):
+def conservative_parameter_merge(
+    params, same_kind=True, same_default=True, same_annotation=True
+):
     """Validates that all the params are exactly the same, returning the first is so."""
     first_param, *_ = params
     if not all(p.name == first_param.name for p in params):
         raise ValidationError(f"Some params didn't have the same name: {params}")
-    if not all(p.kind == first_param.kind for p in params):
+    if same_kind and not all(p.kind == first_param.kind for p in params):
         raise ValidationError(f"Some params didn't have the same kind: {params}")
-    if not all(p.default == first_param.default for p in params):
+    if same_default and not all(p.default == first_param.default for p in params):
         raise ValidationError(f"Some params didn't have the same default: {params}")
-    if not all(p.annotation == first_param.annotation for p in params):
+    if same_annotation and not all(
+        p.annotation == first_param.annotation for p in params
+    ):
         raise ValidationError(f"Some params didn't have the same annotation: {params}")
     return first_param
 
@@ -626,10 +630,14 @@ class DAG:
 
     """
 
-    func_nodes: Iterable[FuncNode]
-    cache_last_scope: bool = True
-    parameter_merge: ParameterMerger = conservative_parameter_merge
-    new_scope: Callable = dict  # can return a prepopulated scope too!
+    func_nodes: Iterable[Union[FuncNode, Callable]]
+    cache_last_scope: bool = field(default=True, repr=False)
+    parameter_merge: ParameterMerger = field(
+        default=conservative_parameter_merge, repr=False
+    )
+    # can return a prepopulated scope too!
+    new_scope: Callable = field(default=dict, repr=False)
+    name: str = None
 
     def __post_init__(self):
         self.func_nodes = tuple(_mk_func_nodes(self.func_nodes))
@@ -637,15 +645,19 @@ class DAG:
         self.nodes = topological_sort(self.graph)
         # reorder the nodes to fit topological order
         self.func_nodes, self.var_nodes = _separate_func_nodes_and_var_nodes(self.nodes)
-        # figure out the roots and leaves
-        self.roots = set(root_nodes(self.graph))
-        self.leafs = set(leaf_nodes(self.graph))
         # self.sig = Sig(dict(extract_items(sig.parameters, 'xz')))
-        self.sig = Sig(sort_params(self.src_name_params(self.roots)))
+        self.sig = Sig(sort_params(self.src_name_params(root_nodes(self.graph))))
         self.sig(self)  # to put the signature on the callable DAG
+        # figure out the roots and leaves
+        self.roots = tuple(self.sig.names)  # roots in the same order as signature
+        leafs = leaf_nodes(self.graph)
+        # But we want leafs in topological order
+        self.leafs = tuple([name for name in self.nodes if name in leafs])
         self.last_scope = None
+        if self.name is not None:
+            self.__name__ = self.name
 
-    def __call__(self, *args, **kwargs):
+    def _call(self, *args, **kwargs):
         scope = self.sig.kwargs_from_args_and_kwargs(args, kwargs)
         self.call_on_scope(scope)
         tup = tuple(extract_values(scope, self.leafs))
@@ -656,6 +668,9 @@ class DAG:
         else:
             return None
 
+    def __call__(self, *args, **kwargs):
+        return self._call(*args, **kwargs)
+
     def call_on_scope(self, scope=None):
         """Calls the func_nodes using scope (a dict or MutableMapping) both to
         source it's arguments and write it's results.
@@ -786,8 +801,8 @@ class DAG:
         return '\n'.join(func_node.synopsis_string() for func_node in self.func_nodes)
 
     # TODO: Give more control (merge with lined)
-    def dot_digraph_body(self):
-        yield from dot_lines_of_func_nodes(self.func_nodes)
+    def dot_digraph_body(self, start_lines=()):
+        yield from dot_lines_of_func_nodes(self.func_nodes, start_lines=start_lines)
 
     @wraps(dot_digraph_body)
     def dot_digraph_ascii(self, *args, **kwargs):
@@ -870,7 +885,7 @@ def _parameters_and_names_from_sig(
     return sig.parameters, out, func_name
 
 
-def dot_lines_of_func_nodes(func_nodes: Iterable[FuncNode]):
+def dot_lines_of_func_nodes(func_nodes: Iterable[FuncNode], start_lines=()):
     r"""Got lines generator for the graphviz.DiGraph(body=list(...))
 
     >>> def add(a, b=1):
@@ -953,6 +968,7 @@ def dot_lines_of_func_nodes(func_nodes: Iterable[FuncNode]):
     <BLANKLINE>
 
     """
+    yield from start_lines
     validate_that_func_node_names_are_sane(func_nodes)
     for func_node in func_nodes:
         yield from dot_lines_of_func_node(func_node)
